==> aiohttp_apiset-0.1.2.tar.gz: aiohttp_apiset-0.1.2/aiohttp_apiset/views.py <==
import asyncio
import inspect
import json
import os
import sys

from aiohttp import web, abc
from aiohttp.web import Application
import yaml

from .swagger.loader import SwaggerLoaderMixin
from . import utils

PY_35 = sys.version_info >= (3, 5)


class BaseApiSet:
    @classmethod
    def append_routes_to(cls, app: Application, prefix=None):
        raise NotImplementedError()


def swagger_yaml(file_path, *, executor=None, loop=None):
    if loop is None:
        loop = asyncio.get_event_loop()

    def response(request):
        with open(file_path) as f:
            data = yaml.load(f)
            return ApiSet.response_json(data)

    @asyncio.coroutine
    def aresponse(request):
        return (
            yield from loop.run_in_executor(
                executor, response, request))

    return aresponse


class ApiSet(abc.AbstractView, BaseApiSet, SwaggerLoaderMixin):
    namespace = NotImplemented
    root_dir = '/'
    swagger_ref = None
    default_response = 'json'
    methods = {
        '': (
            ('options', 'OPTIONS'),
            ('create', 'POST'),
            ('post', 'POST'),
            ('put', 'PUT'),
            ('patch', 'PATCH'),
            ('list', 'GET'),
            ('get', 'GET'),
        ),
        '/{id}': (
            ('retrieve', 'GET'),
            ('delete', 'DELETE'),
        ),
    }
    prefix = None

    @classmethod
    def factory(cls, prefi):
        class View(cls):
            prefix = prefi
        return View

    def __init__(self, request, *, prefix=None):
        super().__init__(request)
        if prefix is not None:
            self.prefix = prefix
        self._methods = {}
        self._postfixes = sorted(self.methods, key=len, reverse=True)
        for pref, methods in self.methods.items():
            meths = self._methods[pref] = {}
            for name, mt in methods:
                if hasattr(self, name):
                    meths[mt] = name

    @asyncio.coroutine
    def __iter__(self):
        if self.prefix is not None:
            postfix = self.request.path[len(self.prefix):]
            if postfix not in self._methods:
                raise web.HTTPMethodNotAllowed(self.request.method, ())
        else:
            for postfix in self._postfixes:
                if self.request.path.endswith(postfix):
                    break
            else:
                raise web.HTTPMethodNotAllowed(self.request.method, ())

        methods = self._methods[postfix]
        if self.request.method not in methods:
            raise web.HTTPMethodNotAllowed(
                self.request.method, tuple(methods))
        method_name = methods[self.request.method.upper()]
        method = getattr(self, method_name)

        params = inspect.signature(method).parameters
        kwargs = {}
        if 'request' in params:
            kwargs['request'] = self.request
        for k in params:
            if k in self.request.match_info:
                kwargs[k] = self.request.match_info[k]

        resp = yield from method(**kwargs)
        return resp

    if PY_35:
        def __await__(self):
            return (yield from self.__iter__())

    @classmethod
    def add_routes(cls, routes: list, prefix):
        basePath = cls.get_sub_swagger('basePath', default='')
        prefix += basePath
        view = cls.factory(prefix)
        for postfix in cls.methods:
            name = utils.to_name(cls.namespace + postfix)
            routes.append((prefix + postfix, view, name))

    @classmethod
    def get_swagger_paths(cls):
        return cls.get_sub_swagger('paths')

    @property
    def loop(self):
        return self.app.loop

    @asyncio.coroutine
    def options(self, request):
        return web.Response(body=b'')

    @asyncio.coroutine
    def data(self, request) -> dict:
        if 'json' in request.content_type:
            data = yield from request.json()
        else:
            data = yield from request.post()
        return data

    @asyncio.coroutine
    def swagger(self, request):
        module_path = sys.modules[self.__module__].__file__
        swagger_file = os.path.join(
            os.path.dirname(module_path), self.swagger_file)
        return (yield from swagger_yaml(swagger_file)(request))

    def add_swagger_route(self, router, prefix=None, namespace=None):
        router.add_route(
            'GET',
            prefix + str(self.version) + self.docs + self.namespace,
            self.swagger,
            name=':'.join((namespace, 'swagger')),
        )

    def add_action_routes(self, router, prefix=None, namespace=None):
        url = prefix + str(self.version) + '/' + self.namespace + '/'
        for action_name, method, postfix_url in self.actions:
            action = getattr(self, action_name, None)
            if action:
                if method == 'OPTIONS':
                    name = None
                else:
                    name = ':'.join((namespace, action_name))
                router.add_route(method, url + postfix_url, action, name=name)

    def get_routes(self, base_url, namespace=None):
        if not namespace:
            namespace = namespace or self.namespace
            namespace = namespace.replace('/', '.')

        for action_name, method, postfix_url in self.actions:
            action = getattr(self, action_name, None)
            if action:
                yield {
                    'method': method,
                    'path': base_url + postfix_url,
                    'handler': action,
                    'name': ':'.join((namespace, action_name)),
                    'swagger_path': self.swagger_path,
                }

    @classmethod
    def append_routes_to(cls, app: Application, prefix=None):
        self = cls(app)
        prefix = prefix or self.prefix
        namespace = self.namespace.replace('/', '.')
        namespace = namespace.replace('{', '')
        namespace = namespace.replace('}', '')
        self.add_swagger_route(app.router, prefix, namespace)
        self.add_action_routes(app.router, prefix, namespace)

    def response(self, data=None, **kwargs):
        if isinstance(data, dict):
            data = data.copy()
        elif not data:
            data = {}
        else:
            data = {'data_text': str(data)}
        data.update(kwargs)
        return getattr(
            self,
            'response_' + self.default_response,
        )(data, **kwargs)

    @classmethod
    def response_json(cls, data, **kwargs):
        data = json.dumps(
            data,
            indent=3,
            ensure_ascii=False,
        )
        return web.Response(
            body=data.encode(),
            content_type='application/json; charset=utf-8',
            status=kwargs.get('status', 200),
        )

==> aiohttp_apiset-0.1.5.tar.gz: aiohttp_apiset-0.1.5/aiohttp_apiset/views.py <==
import asyncio
import inspect
import json
import os
import sys

from aiohttp import web, abc
from aiohttp.web import Application
import yaml

from .swagger.loader import SwaggerLoaderMixin
from . import utils

PY_35 = sys.version_info >= (3, 5)


class BaseApiSet:
    @classmethod
    def append_routes_to(cls, app: Application, prefix=None):
        raise NotImplementedError()


def swagger_yaml(file_path, *, executor=None, loop=None):
    if loop is None:
        loop = asyncio.get_event_loop()

    def response(request):
        with open(file_path) as f:
            data = yaml.load(f)
            return ApiSet.response_json(data)

    @asyncio.coroutine
    def aresponse(request):
        return (
            yield from loop.run_in_executor(
                executor, response, request))

    return aresponse


class ApiSet(abc.AbstractView, BaseApiSet, SwaggerLoaderMixin):
    namespace = NotImplemented
    root_dir = '/'
    swagger_ref = None
    default_response = 'json'
    methods = {
        '': (
            ('options', 'OPTIONS'),
            ('create', 'POST'),
            ('post', 'POST'),
            ('put', 'PUT'),
            ('patch', 'PATCH'),
            ('list', 'GET'),
            ('get', 'GET'),
        ),
        '/{id}': (
            ('retrieve', 'GET'),
            ('delete', 'DELETE'),
        ),
    }
    prefix = None

    @classmethod
    def factory(cls, prefi):
        class View(cls):
            prefix = prefi
        return View

    def __init__(self, request, *, prefix=None):
        super().__init__(request)
        if prefix is not None:
            self.prefix = prefix
        self._methods = {}
        self._postfixes = sorted(self.methods, key=len, reverse=True)
        for pref, methods in self.methods.items():
            meths = self._methods[pref] = {}
            for name, mt in methods:
                if hasattr(self, name):
                    meths[mt] = name

    @asyncio.coroutine
    def __iter__(self):
        if self.prefix is not None:
            postfix = self.request.path[len(self.prefix):]
            if postfix not in self._methods:
                raise web.HTTPMethodNotAllowed(self.request.method, ())
        else:
            for postfix in self._postfixes:
                if self.request.path.endswith(postfix):
                    break
            else:
                raise web.HTTPMethodNotAllowed(self.request.method, ())

        methods = self._methods[postfix]
        if self.request.method not in methods:
            raise web.HTTPMethodNotAllowed(
                self.request.method, tuple(methods))
        method_name = methods[self.request.method.upper()]
        method = getattr(self, method_name)

        params = inspect.signature(method).parameters
        kwargs = {}
        if 'request' in params:
            kwargs['request'] = self.request
        for k in params:
            if k in self.request.match_info:
                kwargs[k] = self.request.match_info[k]

        resp = yield from method(**kwargs)
        return resp

    if PY_35:
        def __await__(self):
            return (yield from self.__iter__())

    @classmethod
    def add_routes(cls, routes: list, prefix):
        basePath = cls.get_sub_swagger('basePath', default='')
        prefix += basePath
        view = cls.factory(prefix)
        for postfix in cls.methods:
            name = utils.to_name(cls.namespace + postfix)
            routes.append(('*', prefix + postfix, view, name))

    @classmethod
    def get_swagger_paths(cls):
        return cls.get_sub_swagger('paths')

    @property
    def loop(self):
        return self.app.loop

    @asyncio.coroutine
    def options(self, request):
        return web.Response(body=b'')

    @asyncio.coroutine
    def data(self, request) -> dict:
        if 'json' in request.content_type:
            data = yield from request.json()
        else:
            data = yield from request.post()
        return data

    @asyncio.coroutine
    def swagger(self, request):
        module_path = sys.modules[self.__module__].__file__
        swagger_file = os.path.join(
            os.path.dirname(module_path), self.swagger_file)
        return (yield from swagger_yaml(swagger_file)(request))

    def add_swagger_route(self, router, prefix=None, namespace=None):
        router.add_route(
            'GET',
            prefix + str(self.version) + self.docs + self.namespace,
            self.swagger,
            name=':'.join((namespace, 'swagger')),
        )

    def add_action_routes(self, router, prefix=None, namespace=None):
        url = prefix + str(self.version) + '/' + self.namespace + '/'
        for action_name, method, postfix_url in self.actions:
            action = getattr(self, action_name, None)
            if action:
                if method == 'OPTIONS':
                    name = None
                else:
                    name = ':'.join((namespace, action_name))
                router.add_route(method, url + postfix_url, action, name=name)

    def get_routes(self, base_url, namespace=None):
        if not namespace:
            namespace = namespace or self.namespace
            namespace = namespace.replace('/', '.')

        for action_name, method, postfix_url in self.actions:
            action = getattr(self, action_name, None)
            if action:
                yield {
                    'method': method,
                    'path': base_url + postfix_url,
                    'handler': action,
                    'name': ':'.join((namespace, action_name)),
                    'swagger_path': self.swagger_path,
                }

    @classmethod
    def append_routes_to(cls, app: Application, prefix=None):
        self = cls(app)
        prefix = prefix or self.prefix
        namespace = self.namespace.replace('/', '.')
        namespace = namespace.replace('{', '')
        namespace = namespace.replace('}', '')
        self.add_swagger_route(app.router, prefix, namespace)
        self.add_action_routes(app.router, prefix, namespace)

    def response(self, data=None, **kwargs):
        if isinstance(data, dict):
            data = data.copy()
        elif not data:
            data = {}
        else:
            data = {'data_text': str(data)}
        data.update(kwargs)
        return getattr(
            self,
            'response_' + self.default_response,
        )(data, **kwargs)

    @classmethod
    def response_json(cls, data, **kwargs):
        data = json.dumps(
            data,
            indent=3,
            ensure_ascii=False,
        )
        return web.Response(
            body=data.encode(),
            content_type='application/json; charset=utf-8',
            status=kwargs.get('status', 200),
        )
